# -----------------------------------------------------------------------------
#
# Name: d*.tst
#
# Version: 1.0
#
# Creator: miyamoto
#
# Date: 980205
#
# Purpose:
#   Sanity test for domains.  It will create the necessary files for an
#   upper level tester to create an executable, run it, and compare the
#   output to an expected output.
#
# Change Log:
#
# -----------------------------------------------------------------------------

cat > test.ti << '@eof'
class testShrink{

public static void printPoint(Point<3> p){
  System.out.print("[");
  for (int i = 1; i <= 3; i++){
    System.out.print(p[i]);
    if (i < 3){
      System.out.print(",");
    }
  }
  System.out.println("]");
}

public static void printDomain(Domain<3> d){
  System.out.println("Type: Domain<3>");
  foreach(p within d){
    printPoint(p);
  }//end foreach
}

public static void printDomain(RectDomain<3> rd){
  System.out.println("Type RectDomain<3>");
  foreach(p within rd){
    printPoint(p);
  }//end foreach
  //  printDomain((Domain<3>) rd); //it's small enough not to need this...
}

public static void main(String argv[]){

  Point<3> lb = [0,0,0];
  Point<3> ub = Point<3>.all(2);

  RectDomain<3> rd = [lb:ub];
  RectDomain<3> rd1;

  System.out.println("Points in rd");
  printDomain(rd);

  System.out.println("rd.accrete(3, 2, 1)");
  printDomain(rd.accrete(3,2,1));

  System.out.println("rd.accrete(3,Point<3>.all(1))");
  printDomain(rd.accrete(3,Point<3>.all(1)));

  System.out.println("rd.accrete(3, 1, 1)...");
  rd1 = rd.accrete(3, 1, 1);
  rd1 = rd1.accrete(3, -1, 1);
  rd1 = rd1.accrete(3, 2, 1);
  rd1 = rd1.accrete(3, -2, 1);
  rd1 = rd1.accrete(3, 3, 1);
  rd1 = rd1.accrete(3, -3, 1);
  printDomain(rd1);
  
  System.out.println("rd.shrink(1, 2)");
  printDomain(rd.shrink(1,2));

  System.out.println("rd.shrink(1)");
  printDomain(rd.shrink(1));

}//end main

}//end class

@eof

cat > stdout.old << '@eof'
Points in rd
Type RectDomain<3>
[0,0,0]
[0,0,1]
[0,0,2]
[0,1,0]
[0,1,1]
[0,1,2]
[0,2,0]
[0,2,1]
[0,2,2]
[1,0,0]
[1,0,1]
[1,0,2]
[1,1,0]
[1,1,1]
[1,1,2]
[1,2,0]
[1,2,1]
[1,2,2]
[2,0,0]
[2,0,1]
[2,0,2]
[2,1,0]
[2,1,1]
[2,1,2]
[2,2,0]
[2,2,1]
[2,2,2]
rd.accrete(3, 2, 1)
Type RectDomain<3>
[0,0,0]
[0,0,1]
[0,0,2]
[0,1,0]
[0,1,1]
[0,1,2]
[0,2,0]
[0,2,1]
[0,2,2]
[0,3,0]
[0,3,1]
[0,3,2]
[0,4,0]
[0,4,1]
[0,4,2]
[0,5,0]
[0,5,1]
[0,5,2]
[1,0,0]
[1,0,1]
[1,0,2]
[1,1,0]
[1,1,1]
[1,1,2]
[1,2,0]
[1,2,1]
[1,2,2]
[1,3,0]
[1,3,1]
[1,3,2]
[1,4,0]
[1,4,1]
[1,4,2]
[1,5,0]
[1,5,1]
[1,5,2]
[2,0,0]
[2,0,1]
[2,0,2]
[2,1,0]
[2,1,1]
[2,1,2]
[2,2,0]
[2,2,1]
[2,2,2]
[2,3,0]
[2,3,1]
[2,3,2]
[2,4,0]
[2,4,1]
[2,4,2]
[2,5,0]
[2,5,1]
[2,5,2]
rd.accrete(3,Point<3>.all(1))
Type RectDomain<3>
[-3,-3,-3]
[-3,-3,-2]
[-3,-3,-1]
[-3,-3,0]
[-3,-3,1]
[-3,-3,2]
[-3,-3,3]
[-3,-3,4]
[-3,-3,5]
[-3,-2,-3]
[-3,-2,-2]
[-3,-2,-1]
[-3,-2,0]
[-3,-2,1]
[-3,-2,2]
[-3,-2,3]
[-3,-2,4]
[-3,-2,5]
[-3,-1,-3]
[-3,-1,-2]
[-3,-1,-1]
[-3,-1,0]
[-3,-1,1]
[-3,-1,2]
[-3,-1,3]
[-3,-1,4]
[-3,-1,5]
[-3,0,-3]
[-3,0,-2]
[-3,0,-1]
[-3,0,0]
[-3,0,1]
[-3,0,2]
[-3,0,3]
[-3,0,4]
[-3,0,5]
[-3,1,-3]
[-3,1,-2]
[-3,1,-1]
[-3,1,0]
[-3,1,1]
[-3,1,2]
[-3,1,3]
[-3,1,4]
[-3,1,5]
[-3,2,-3]
[-3,2,-2]
[-3,2,-1]
[-3,2,0]
[-3,2,1]
[-3,2,2]
[-3,2,3]
[-3,2,4]
[-3,2,5]
[-3,3,-3]
[-3,3,-2]
[-3,3,-1]
[-3,3,0]
[-3,3,1]
[-3,3,2]
[-3,3,3]
[-3,3,4]
[-3,3,5]
[-3,4,-3]
[-3,4,-2]
[-3,4,-1]
[-3,4,0]
[-3,4,1]
[-3,4,2]
[-3,4,3]
[-3,4,4]
[-3,4,5]
[-3,5,-3]
[-3,5,-2]
[-3,5,-1]
[-3,5,0]
[-3,5,1]
[-3,5,2]
[-3,5,3]
[-3,5,4]
[-3,5,5]
[-2,-3,-3]
[-2,-3,-2]
[-2,-3,-1]
[-2,-3,0]
[-2,-3,1]
[-2,-3,2]
[-2,-3,3]
[-2,-3,4]
[-2,-3,5]
[-2,-2,-3]
[-2,-2,-2]
[-2,-2,-1]
[-2,-2,0]
[-2,-2,1]
[-2,-2,2]
[-2,-2,3]
[-2,-2,4]
[-2,-2,5]
[-2,-1,-3]
[-2,-1,-2]
[-2,-1,-1]
[-2,-1,0]
[-2,-1,1]
[-2,-1,2]
[-2,-1,3]
[-2,-1,4]
[-2,-1,5]
[-2,0,-3]
[-2,0,-2]
[-2,0,-1]
[-2,0,0]
[-2,0,1]
[-2,0,2]
[-2,0,3]
[-2,0,4]
[-2,0,5]
[-2,1,-3]
[-2,1,-2]
[-2,1,-1]
[-2,1,0]
[-2,1,1]
[-2,1,2]
[-2,1,3]
[-2,1,4]
[-2,1,5]
[-2,2,-3]
[-2,2,-2]
[-2,2,-1]
[-2,2,0]
[-2,2,1]
[-2,2,2]
[-2,2,3]
[-2,2,4]
[-2,2,5]
[-2,3,-3]
[-2,3,-2]
[-2,3,-1]
[-2,3,0]
[-2,3,1]
[-2,3,2]
[-2,3,3]
[-2,3,4]
[-2,3,5]
[-2,4,-3]
[-2,4,-2]
[-2,4,-1]
[-2,4,0]
[-2,4,1]
[-2,4,2]
[-2,4,3]
[-2,4,4]
[-2,4,5]
[-2,5,-3]
[-2,5,-2]
[-2,5,-1]
[-2,5,0]
[-2,5,1]
[-2,5,2]
[-2,5,3]
[-2,5,4]
[-2,5,5]
[-1,-3,-3]
[-1,-3,-2]
[-1,-3,-1]
[-1,-3,0]
[-1,-3,1]
[-1,-3,2]
[-1,-3,3]
[-1,-3,4]
[-1,-3,5]
[-1,-2,-3]
[-1,-2,-2]
[-1,-2,-1]
[-1,-2,0]
[-1,-2,1]
[-1,-2,2]
[-1,-2,3]
[-1,-2,4]
[-1,-2,5]
[-1,-1,-3]
[-1,-1,-2]
[-1,-1,-1]
[-1,-1,0]
[-1,-1,1]
[-1,-1,2]
[-1,-1,3]
[-1,-1,4]
[-1,-1,5]
[-1,0,-3]
[-1,0,-2]
[-1,0,-1]
[-1,0,0]
[-1,0,1]
[-1,0,2]
[-1,0,3]
[-1,0,4]
[-1,0,5]
[-1,1,-3]
[-1,1,-2]
[-1,1,-1]
[-1,1,0]
[-1,1,1]
[-1,1,2]
[-1,1,3]
[-1,1,4]
[-1,1,5]
[-1,2,-3]
[-1,2,-2]
[-1,2,-1]
[-1,2,0]
[-1,2,1]
[-1,2,2]
[-1,2,3]
[-1,2,4]
[-1,2,5]
[-1,3,-3]
[-1,3,-2]
[-1,3,-1]
[-1,3,0]
[-1,3,1]
[-1,3,2]
[-1,3,3]
[-1,3,4]
[-1,3,5]
[-1,4,-3]
[-1,4,-2]
[-1,4,-1]
[-1,4,0]
[-1,4,1]
[-1,4,2]
[-1,4,3]
[-1,4,4]
[-1,4,5]
[-1,5,-3]
[-1,5,-2]
[-1,5,-1]
[-1,5,0]
[-1,5,1]
[-1,5,2]
[-1,5,3]
[-1,5,4]
[-1,5,5]
[0,-3,-3]
[0,-3,-2]
[0,-3,-1]
[0,-3,0]
[0,-3,1]
[0,-3,2]
[0,-3,3]
[0,-3,4]
[0,-3,5]
[0,-2,-3]
[0,-2,-2]
[0,-2,-1]
[0,-2,0]
[0,-2,1]
[0,-2,2]
[0,-2,3]
[0,-2,4]
[0,-2,5]
[0,-1,-3]
[0,-1,-2]
[0,-1,-1]
[0,-1,0]
[0,-1,1]
[0,-1,2]
[0,-1,3]
[0,-1,4]
[0,-1,5]
[0,0,-3]
[0,0,-2]
[0,0,-1]
[0,0,0]
[0,0,1]
[0,0,2]
[0,0,3]
[0,0,4]
[0,0,5]
[0,1,-3]
[0,1,-2]
[0,1,-1]
[0,1,0]
[0,1,1]
[0,1,2]
[0,1,3]
[0,1,4]
[0,1,5]
[0,2,-3]
[0,2,-2]
[0,2,-1]
[0,2,0]
[0,2,1]
[0,2,2]
[0,2,3]
[0,2,4]
[0,2,5]
[0,3,-3]
[0,3,-2]
[0,3,-1]
[0,3,0]
[0,3,1]
[0,3,2]
[0,3,3]
[0,3,4]
[0,3,5]
[0,4,-3]
[0,4,-2]
[0,4,-1]
[0,4,0]
[0,4,1]
[0,4,2]
[0,4,3]
[0,4,4]
[0,4,5]
[0,5,-3]
[0,5,-2]
[0,5,-1]
[0,5,0]
[0,5,1]
[0,5,2]
[0,5,3]
[0,5,4]
[0,5,5]
[1,-3,-3]
[1,-3,-2]
[1,-3,-1]
[1,-3,0]
[1,-3,1]
[1,-3,2]
[1,-3,3]
[1,-3,4]
[1,-3,5]
[1,-2,-3]
[1,-2,-2]
[1,-2,-1]
[1,-2,0]
[1,-2,1]
[1,-2,2]
[1,-2,3]
[1,-2,4]
[1,-2,5]
[1,-1,-3]
[1,-1,-2]
[1,-1,-1]
[1,-1,0]
[1,-1,1]
[1,-1,2]
[1,-1,3]
[1,-1,4]
[1,-1,5]
[1,0,-3]
[1,0,-2]
[1,0,-1]
[1,0,0]
[1,0,1]
[1,0,2]
[1,0,3]
[1,0,4]
[1,0,5]
[1,1,-3]
[1,1,-2]
[1,1,-1]
[1,1,0]
[1,1,1]
[1,1,2]
[1,1,3]
[1,1,4]
[1,1,5]
[1,2,-3]
[1,2,-2]
[1,2,-1]
[1,2,0]
[1,2,1]
[1,2,2]
[1,2,3]
[1,2,4]
[1,2,5]
[1,3,-3]
[1,3,-2]
[1,3,-1]
[1,3,0]
[1,3,1]
[1,3,2]
[1,3,3]
[1,3,4]
[1,3,5]
[1,4,-3]
[1,4,-2]
[1,4,-1]
[1,4,0]
[1,4,1]
[1,4,2]
[1,4,3]
[1,4,4]
[1,4,5]
[1,5,-3]
[1,5,-2]
[1,5,-1]
[1,5,0]
[1,5,1]
[1,5,2]
[1,5,3]
[1,5,4]
[1,5,5]
[2,-3,-3]
[2,-3,-2]
[2,-3,-1]
[2,-3,0]
[2,-3,1]
[2,-3,2]
[2,-3,3]
[2,-3,4]
[2,-3,5]
[2,-2,-3]
[2,-2,-2]
[2,-2,-1]
[2,-2,0]
[2,-2,1]
[2,-2,2]
[2,-2,3]
[2,-2,4]
[2,-2,5]
[2,-1,-3]
[2,-1,-2]
[2,-1,-1]
[2,-1,0]
[2,-1,1]
[2,-1,2]
[2,-1,3]
[2,-1,4]
[2,-1,5]
[2,0,-3]
[2,0,-2]
[2,0,-1]
[2,0,0]
[2,0,1]
[2,0,2]
[2,0,3]
[2,0,4]
[2,0,5]
[2,1,-3]
[2,1,-2]
[2,1,-1]
[2,1,0]
[2,1,1]
[2,1,2]
[2,1,3]
[2,1,4]
[2,1,5]
[2,2,-3]
[2,2,-2]
[2,2,-1]
[2,2,0]
[2,2,1]
[2,2,2]
[2,2,3]
[2,2,4]
[2,2,5]
[2,3,-3]
[2,3,-2]
[2,3,-1]
[2,3,0]
[2,3,1]
[2,3,2]
[2,3,3]
[2,3,4]
[2,3,5]
[2,4,-3]
[2,4,-2]
[2,4,-1]
[2,4,0]
[2,4,1]
[2,4,2]
[2,4,3]
[2,4,4]
[2,4,5]
[2,5,-3]
[2,5,-2]
[2,5,-1]
[2,5,0]
[2,5,1]
[2,5,2]
[2,5,3]
[2,5,4]
[2,5,5]
[3,-3,-3]
[3,-3,-2]
[3,-3,-1]
[3,-3,0]
[3,-3,1]
[3,-3,2]
[3,-3,3]
[3,-3,4]
[3,-3,5]
[3,-2,-3]
[3,-2,-2]
[3,-2,-1]
[3,-2,0]
[3,-2,1]
[3,-2,2]
[3,-2,3]
[3,-2,4]
[3,-2,5]
[3,-1,-3]
[3,-1,-2]
[3,-1,-1]
[3,-1,0]
[3,-1,1]
[3,-1,2]
[3,-1,3]
[3,-1,4]
[3,-1,5]
[3,0,-3]
[3,0,-2]
[3,0,-1]
[3,0,0]
[3,0,1]
[3,0,2]
[3,0,3]
[3,0,4]
[3,0,5]
[3,1,-3]
[3,1,-2]
[3,1,-1]
[3,1,0]
[3,1,1]
[3,1,2]
[3,1,3]
[3,1,4]
[3,1,5]
[3,2,-3]
[3,2,-2]
[3,2,-1]
[3,2,0]
[3,2,1]
[3,2,2]
[3,2,3]
[3,2,4]
[3,2,5]
[3,3,-3]
[3,3,-2]
[3,3,-1]
[3,3,0]
[3,3,1]
[3,3,2]
[3,3,3]
[3,3,4]
[3,3,5]
[3,4,-3]
[3,4,-2]
[3,4,-1]
[3,4,0]
[3,4,1]
[3,4,2]
[3,4,3]
[3,4,4]
[3,4,5]
[3,5,-3]
[3,5,-2]
[3,5,-1]
[3,5,0]
[3,5,1]
[3,5,2]
[3,5,3]
[3,5,4]
[3,5,5]
[4,-3,-3]
[4,-3,-2]
[4,-3,-1]
[4,-3,0]
[4,-3,1]
[4,-3,2]
[4,-3,3]
[4,-3,4]
[4,-3,5]
[4,-2,-3]
[4,-2,-2]
[4,-2,-1]
[4,-2,0]
[4,-2,1]
[4,-2,2]
[4,-2,3]
[4,-2,4]
[4,-2,5]
[4,-1,-3]
[4,-1,-2]
[4,-1,-1]
[4,-1,0]
[4,-1,1]
[4,-1,2]
[4,-1,3]
[4,-1,4]
[4,-1,5]
[4,0,-3]
[4,0,-2]
[4,0,-1]
[4,0,0]
[4,0,1]
[4,0,2]
[4,0,3]
[4,0,4]
[4,0,5]
[4,1,-3]
[4,1,-2]
[4,1,-1]
[4,1,0]
[4,1,1]
[4,1,2]
[4,1,3]
[4,1,4]
[4,1,5]
[4,2,-3]
[4,2,-2]
[4,2,-1]
[4,2,0]
[4,2,1]
[4,2,2]
[4,2,3]
[4,2,4]
[4,2,5]
[4,3,-3]
[4,3,-2]
[4,3,-1]
[4,3,0]
[4,3,1]
[4,3,2]
[4,3,3]
[4,3,4]
[4,3,5]
[4,4,-3]
[4,4,-2]
[4,4,-1]
[4,4,0]
[4,4,1]
[4,4,2]
[4,4,3]
[4,4,4]
[4,4,5]
[4,5,-3]
[4,5,-2]
[4,5,-1]
[4,5,0]
[4,5,1]
[4,5,2]
[4,5,3]
[4,5,4]
[4,5,5]
[5,-3,-3]
[5,-3,-2]
[5,-3,-1]
[5,-3,0]
[5,-3,1]
[5,-3,2]
[5,-3,3]
[5,-3,4]
[5,-3,5]
[5,-2,-3]
[5,-2,-2]
[5,-2,-1]
[5,-2,0]
[5,-2,1]
[5,-2,2]
[5,-2,3]
[5,-2,4]
[5,-2,5]
[5,-1,-3]
[5,-1,-2]
[5,-1,-1]
[5,-1,0]
[5,-1,1]
[5,-1,2]
[5,-1,3]
[5,-1,4]
[5,-1,5]
[5,0,-3]
[5,0,-2]
[5,0,-1]
[5,0,0]
[5,0,1]
[5,0,2]
[5,0,3]
[5,0,4]
[5,0,5]
[5,1,-3]
[5,1,-2]
[5,1,-1]
[5,1,0]
[5,1,1]
[5,1,2]
[5,1,3]
[5,1,4]
[5,1,5]
[5,2,-3]
[5,2,-2]
[5,2,-1]
[5,2,0]
[5,2,1]
[5,2,2]
[5,2,3]
[5,2,4]
[5,2,5]
[5,3,-3]
[5,3,-2]
[5,3,-1]
[5,3,0]
[5,3,1]
[5,3,2]
[5,3,3]
[5,3,4]
[5,3,5]
[5,4,-3]
[5,4,-2]
[5,4,-1]
[5,4,0]
[5,4,1]
[5,4,2]
[5,4,3]
[5,4,4]
[5,4,5]
[5,5,-3]
[5,5,-2]
[5,5,-1]
[5,5,0]
[5,5,1]
[5,5,2]
[5,5,3]
[5,5,4]
[5,5,5]
rd.accrete(3, 1, 1)...
Type RectDomain<3>
[-3,-3,-3]
[-3,-3,-2]
[-3,-3,-1]
[-3,-3,0]
[-3,-3,1]
[-3,-3,2]
[-3,-3,3]
[-3,-3,4]
[-3,-3,5]
[-3,-2,-3]
[-3,-2,-2]
[-3,-2,-1]
[-3,-2,0]
[-3,-2,1]
[-3,-2,2]
[-3,-2,3]
[-3,-2,4]
[-3,-2,5]
[-3,-1,-3]
[-3,-1,-2]
[-3,-1,-1]
[-3,-1,0]
[-3,-1,1]
[-3,-1,2]
[-3,-1,3]
[-3,-1,4]
[-3,-1,5]
[-3,0,-3]
[-3,0,-2]
[-3,0,-1]
[-3,0,0]
[-3,0,1]
[-3,0,2]
[-3,0,3]
[-3,0,4]
[-3,0,5]
[-3,1,-3]
[-3,1,-2]
[-3,1,-1]
[-3,1,0]
[-3,1,1]
[-3,1,2]
[-3,1,3]
[-3,1,4]
[-3,1,5]
[-3,2,-3]
[-3,2,-2]
[-3,2,-1]
[-3,2,0]
[-3,2,1]
[-3,2,2]
[-3,2,3]
[-3,2,4]
[-3,2,5]
[-3,3,-3]
[-3,3,-2]
[-3,3,-1]
[-3,3,0]
[-3,3,1]
[-3,3,2]
[-3,3,3]
[-3,3,4]
[-3,3,5]
[-3,4,-3]
[-3,4,-2]
[-3,4,-1]
[-3,4,0]
[-3,4,1]
[-3,4,2]
[-3,4,3]
[-3,4,4]
[-3,4,5]
[-3,5,-3]
[-3,5,-2]
[-3,5,-1]
[-3,5,0]
[-3,5,1]
[-3,5,2]
[-3,5,3]
[-3,5,4]
[-3,5,5]
[-2,-3,-3]
[-2,-3,-2]
[-2,-3,-1]
[-2,-3,0]
[-2,-3,1]
[-2,-3,2]
[-2,-3,3]
[-2,-3,4]
[-2,-3,5]
[-2,-2,-3]
[-2,-2,-2]
[-2,-2,-1]
[-2,-2,0]
[-2,-2,1]
[-2,-2,2]
[-2,-2,3]
[-2,-2,4]
[-2,-2,5]
[-2,-1,-3]
[-2,-1,-2]
[-2,-1,-1]
[-2,-1,0]
[-2,-1,1]
[-2,-1,2]
[-2,-1,3]
[-2,-1,4]
[-2,-1,5]
[-2,0,-3]
[-2,0,-2]
[-2,0,-1]
[-2,0,0]
[-2,0,1]
[-2,0,2]
[-2,0,3]
[-2,0,4]
[-2,0,5]
[-2,1,-3]
[-2,1,-2]
[-2,1,-1]
[-2,1,0]
[-2,1,1]
[-2,1,2]
[-2,1,3]
[-2,1,4]
[-2,1,5]
[-2,2,-3]
[-2,2,-2]
[-2,2,-1]
[-2,2,0]
[-2,2,1]
[-2,2,2]
[-2,2,3]
[-2,2,4]
[-2,2,5]
[-2,3,-3]
[-2,3,-2]
[-2,3,-1]
[-2,3,0]
[-2,3,1]
[-2,3,2]
[-2,3,3]
[-2,3,4]
[-2,3,5]
[-2,4,-3]
[-2,4,-2]
[-2,4,-1]
[-2,4,0]
[-2,4,1]
[-2,4,2]
[-2,4,3]
[-2,4,4]
[-2,4,5]
[-2,5,-3]
[-2,5,-2]
[-2,5,-1]
[-2,5,0]
[-2,5,1]
[-2,5,2]
[-2,5,3]
[-2,5,4]
[-2,5,5]
[-1,-3,-3]
[-1,-3,-2]
[-1,-3,-1]
[-1,-3,0]
[-1,-3,1]
[-1,-3,2]
[-1,-3,3]
[-1,-3,4]
[-1,-3,5]
[-1,-2,-3]
[-1,-2,-2]
[-1,-2,-1]
[-1,-2,0]
[-1,-2,1]
[-1,-2,2]
[-1,-2,3]
[-1,-2,4]
[-1,-2,5]
[-1,-1,-3]
[-1,-1,-2]
[-1,-1,-1]
[-1,-1,0]
[-1,-1,1]
[-1,-1,2]
[-1,-1,3]
[-1,-1,4]
[-1,-1,5]
[-1,0,-3]
[-1,0,-2]
[-1,0,-1]
[-1,0,0]
[-1,0,1]
[-1,0,2]
[-1,0,3]
[-1,0,4]
[-1,0,5]
[-1,1,-3]
[-1,1,-2]
[-1,1,-1]
[-1,1,0]
[-1,1,1]
[-1,1,2]
[-1,1,3]
[-1,1,4]
[-1,1,5]
[-1,2,-3]
[-1,2,-2]
[-1,2,-1]
[-1,2,0]
[-1,2,1]
[-1,2,2]
[-1,2,3]
[-1,2,4]
[-1,2,5]
[-1,3,-3]
[-1,3,-2]
[-1,3,-1]
[-1,3,0]
[-1,3,1]
[-1,3,2]
[-1,3,3]
[-1,3,4]
[-1,3,5]
[-1,4,-3]
[-1,4,-2]
[-1,4,-1]
[-1,4,0]
[-1,4,1]
[-1,4,2]
[-1,4,3]
[-1,4,4]
[-1,4,5]
[-1,5,-3]
[-1,5,-2]
[-1,5,-1]
[-1,5,0]
[-1,5,1]
[-1,5,2]
[-1,5,3]
[-1,5,4]
[-1,5,5]
[0,-3,-3]
[0,-3,-2]
[0,-3,-1]
[0,-3,0]
[0,-3,1]
[0,-3,2]
[0,-3,3]
[0,-3,4]
[0,-3,5]
[0,-2,-3]
[0,-2,-2]
[0,-2,-1]
[0,-2,0]
[0,-2,1]
[0,-2,2]
[0,-2,3]
[0,-2,4]
[0,-2,5]
[0,-1,-3]
[0,-1,-2]
[0,-1,-1]
[0,-1,0]
[0,-1,1]
[0,-1,2]
[0,-1,3]
[0,-1,4]
[0,-1,5]
[0,0,-3]
[0,0,-2]
[0,0,-1]
[0,0,0]
[0,0,1]
[0,0,2]
[0,0,3]
[0,0,4]
[0,0,5]
[0,1,-3]
[0,1,-2]
[0,1,-1]
[0,1,0]
[0,1,1]
[0,1,2]
[0,1,3]
[0,1,4]
[0,1,5]
[0,2,-3]
[0,2,-2]
[0,2,-1]
[0,2,0]
[0,2,1]
[0,2,2]
[0,2,3]
[0,2,4]
[0,2,5]
[0,3,-3]
[0,3,-2]
[0,3,-1]
[0,3,0]
[0,3,1]
[0,3,2]
[0,3,3]
[0,3,4]
[0,3,5]
[0,4,-3]
[0,4,-2]
[0,4,-1]
[0,4,0]
[0,4,1]
[0,4,2]
[0,4,3]
[0,4,4]
[0,4,5]
[0,5,-3]
[0,5,-2]
[0,5,-1]
[0,5,0]
[0,5,1]
[0,5,2]
[0,5,3]
[0,5,4]
[0,5,5]
[1,-3,-3]
[1,-3,-2]
[1,-3,-1]
[1,-3,0]
[1,-3,1]
[1,-3,2]
[1,-3,3]
[1,-3,4]
[1,-3,5]
[1,-2,-3]
[1,-2,-2]
[1,-2,-1]
[1,-2,0]
[1,-2,1]
[1,-2,2]
[1,-2,3]
[1,-2,4]
[1,-2,5]
[1,-1,-3]
[1,-1,-2]
[1,-1,-1]
[1,-1,0]
[1,-1,1]
[1,-1,2]
[1,-1,3]
[1,-1,4]
[1,-1,5]
[1,0,-3]
[1,0,-2]
[1,0,-1]
[1,0,0]
[1,0,1]
[1,0,2]
[1,0,3]
[1,0,4]
[1,0,5]
[1,1,-3]
[1,1,-2]
[1,1,-1]
[1,1,0]
[1,1,1]
[1,1,2]
[1,1,3]
[1,1,4]
[1,1,5]
[1,2,-3]
[1,2,-2]
[1,2,-1]
[1,2,0]
[1,2,1]
[1,2,2]
[1,2,3]
[1,2,4]
[1,2,5]
[1,3,-3]
[1,3,-2]
[1,3,-1]
[1,3,0]
[1,3,1]
[1,3,2]
[1,3,3]
[1,3,4]
[1,3,5]
[1,4,-3]
[1,4,-2]
[1,4,-1]
[1,4,0]
[1,4,1]
[1,4,2]
[1,4,3]
[1,4,4]
[1,4,5]
[1,5,-3]
[1,5,-2]
[1,5,-1]
[1,5,0]
[1,5,1]
[1,5,2]
[1,5,3]
[1,5,4]
[1,5,5]
[2,-3,-3]
[2,-3,-2]
[2,-3,-1]
[2,-3,0]
[2,-3,1]
[2,-3,2]
[2,-3,3]
[2,-3,4]
[2,-3,5]
[2,-2,-3]
[2,-2,-2]
[2,-2,-1]
[2,-2,0]
[2,-2,1]
[2,-2,2]
[2,-2,3]
[2,-2,4]
[2,-2,5]
[2,-1,-3]
[2,-1,-2]
[2,-1,-1]
[2,-1,0]
[2,-1,1]
[2,-1,2]
[2,-1,3]
[2,-1,4]
[2,-1,5]
[2,0,-3]
[2,0,-2]
[2,0,-1]
[2,0,0]
[2,0,1]
[2,0,2]
[2,0,3]
[2,0,4]
[2,0,5]
[2,1,-3]
[2,1,-2]
[2,1,-1]
[2,1,0]
[2,1,1]
[2,1,2]
[2,1,3]
[2,1,4]
[2,1,5]
[2,2,-3]
[2,2,-2]
[2,2,-1]
[2,2,0]
[2,2,1]
[2,2,2]
[2,2,3]
[2,2,4]
[2,2,5]
[2,3,-3]
[2,3,-2]
[2,3,-1]
[2,3,0]
[2,3,1]
[2,3,2]
[2,3,3]
[2,3,4]
[2,3,5]
[2,4,-3]
[2,4,-2]
[2,4,-1]
[2,4,0]
[2,4,1]
[2,4,2]
[2,4,3]
[2,4,4]
[2,4,5]
[2,5,-3]
[2,5,-2]
[2,5,-1]
[2,5,0]
[2,5,1]
[2,5,2]
[2,5,3]
[2,5,4]
[2,5,5]
[3,-3,-3]
[3,-3,-2]
[3,-3,-1]
[3,-3,0]
[3,-3,1]
[3,-3,2]
[3,-3,3]
[3,-3,4]
[3,-3,5]
[3,-2,-3]
[3,-2,-2]
[3,-2,-1]
[3,-2,0]
[3,-2,1]
[3,-2,2]
[3,-2,3]
[3,-2,4]
[3,-2,5]
[3,-1,-3]
[3,-1,-2]
[3,-1,-1]
[3,-1,0]
[3,-1,1]
[3,-1,2]
[3,-1,3]
[3,-1,4]
[3,-1,5]
[3,0,-3]
[3,0,-2]
[3,0,-1]
[3,0,0]
[3,0,1]
[3,0,2]
[3,0,3]
[3,0,4]
[3,0,5]
[3,1,-3]
[3,1,-2]
[3,1,-1]
[3,1,0]
[3,1,1]
[3,1,2]
[3,1,3]
[3,1,4]
[3,1,5]
[3,2,-3]
[3,2,-2]
[3,2,-1]
[3,2,0]
[3,2,1]
[3,2,2]
[3,2,3]
[3,2,4]
[3,2,5]
[3,3,-3]
[3,3,-2]
[3,3,-1]
[3,3,0]
[3,3,1]
[3,3,2]
[3,3,3]
[3,3,4]
[3,3,5]
[3,4,-3]
[3,4,-2]
[3,4,-1]
[3,4,0]
[3,4,1]
[3,4,2]
[3,4,3]
[3,4,4]
[3,4,5]
[3,5,-3]
[3,5,-2]
[3,5,-1]
[3,5,0]
[3,5,1]
[3,5,2]
[3,5,3]
[3,5,4]
[3,5,5]
[4,-3,-3]
[4,-3,-2]
[4,-3,-1]
[4,-3,0]
[4,-3,1]
[4,-3,2]
[4,-3,3]
[4,-3,4]
[4,-3,5]
[4,-2,-3]
[4,-2,-2]
[4,-2,-1]
[4,-2,0]
[4,-2,1]
[4,-2,2]
[4,-2,3]
[4,-2,4]
[4,-2,5]
[4,-1,-3]
[4,-1,-2]
[4,-1,-1]
[4,-1,0]
[4,-1,1]
[4,-1,2]
[4,-1,3]
[4,-1,4]
[4,-1,5]
[4,0,-3]
[4,0,-2]
[4,0,-1]
[4,0,0]
[4,0,1]
[4,0,2]
[4,0,3]
[4,0,4]
[4,0,5]
[4,1,-3]
[4,1,-2]
[4,1,-1]
[4,1,0]
[4,1,1]
[4,1,2]
[4,1,3]
[4,1,4]
[4,1,5]
[4,2,-3]
[4,2,-2]
[4,2,-1]
[4,2,0]
[4,2,1]
[4,2,2]
[4,2,3]
[4,2,4]
[4,2,5]
[4,3,-3]
[4,3,-2]
[4,3,-1]
[4,3,0]
[4,3,1]
[4,3,2]
[4,3,3]
[4,3,4]
[4,3,5]
[4,4,-3]
[4,4,-2]
[4,4,-1]
[4,4,0]
[4,4,1]
[4,4,2]
[4,4,3]
[4,4,4]
[4,4,5]
[4,5,-3]
[4,5,-2]
[4,5,-1]
[4,5,0]
[4,5,1]
[4,5,2]
[4,5,3]
[4,5,4]
[4,5,5]
[5,-3,-3]
[5,-3,-2]
[5,-3,-1]
[5,-3,0]
[5,-3,1]
[5,-3,2]
[5,-3,3]
[5,-3,4]
[5,-3,5]
[5,-2,-3]
[5,-2,-2]
[5,-2,-1]
[5,-2,0]
[5,-2,1]
[5,-2,2]
[5,-2,3]
[5,-2,4]
[5,-2,5]
[5,-1,-3]
[5,-1,-2]
[5,-1,-1]
[5,-1,0]
[5,-1,1]
[5,-1,2]
[5,-1,3]
[5,-1,4]
[5,-1,5]
[5,0,-3]
[5,0,-2]
[5,0,-1]
[5,0,0]
[5,0,1]
[5,0,2]
[5,0,3]
[5,0,4]
[5,0,5]
[5,1,-3]
[5,1,-2]
[5,1,-1]
[5,1,0]
[5,1,1]
[5,1,2]
[5,1,3]
[5,1,4]
[5,1,5]
[5,2,-3]
[5,2,-2]
[5,2,-1]
[5,2,0]
[5,2,1]
[5,2,2]
[5,2,3]
[5,2,4]
[5,2,5]
[5,3,-3]
[5,3,-2]
[5,3,-1]
[5,3,0]
[5,3,1]
[5,3,2]
[5,3,3]
[5,3,4]
[5,3,5]
[5,4,-3]
[5,4,-2]
[5,4,-1]
[5,4,0]
[5,4,1]
[5,4,2]
[5,4,3]
[5,4,4]
[5,4,5]
[5,5,-3]
[5,5,-2]
[5,5,-1]
[5,5,0]
[5,5,1]
[5,5,2]
[5,5,3]
[5,5,4]
[5,5,5]
rd.shrink(1, 2)
Type RectDomain<3>
[0,0,0]
[0,0,1]
[0,0,2]
[0,1,0]
[0,1,1]
[0,1,2]
[1,0,0]
[1,0,1]
[1,0,2]
[1,1,0]
[1,1,1]
[1,1,2]
[2,0,0]
[2,0,1]
[2,0,2]
[2,1,0]
[2,1,1]
[2,1,2]
rd.shrink(1)
Type RectDomain<3>
[1,1,1]
@eof

cat > stderr.old << '@eof'
@eof
