/* pointers.h - Make global pointers happen. Compiler specific stuff. */
/* see copyright.txt for usage terms */

#ifndef __POINTERS_H
#define __POINTERS_H

/* Mhh, do we really want to do this? -TvE */
#ifndef NULL
#  define NULL (0)
#endif

/* 
 * Global pointer representation now lives in primitives.h
 */

#include <primitives.h>


/* .............................
 *
 * Constructing global addresses
 *
 * .............................
 */

/* Creates a global pointer which is proc:local */
INLINE_MODIFIER jGPointer toglobal(Process proc, void *local)
INLINE({
	jGPointer tempPtr;

	SET_ADDR(tempPtr, local);
	SET_PROC(tempPtr, proc);
	SET_BOX(tempPtr, COMM_GetBoxNumberForProcNumber(proc));
	return tempPtr;
})

/* Creates a global pointer which is proc:local */
INLINE_MODIFIER jGPointer toglobalwithbox(Process proc, Process box, 
					void *local)
INLINE({
	jGPointer tempPtr;

	SET_ADDR(tempPtr, local);
	SET_PROC(tempPtr, proc);
	SET_BOX(tempPtr, box);
	return tempPtr;
})

/* Mutates a global pointer which is proc:local */
#define mutateglobal(globPtr, proc, local) \
  do {\
    SET_ADDR(globPtr, local);\
    SET_PROC(globPtr, proc);\
    SET_BOX(globPtr, COMM_GetBoxNumberForProcNumber(proc)); \
} while(0)

/* Mutates a global pointer which is proc:local */
#define mutateglobalwithbox(globPtr, proc, box, local) \
  do {   \
	SET_ADDR(globPtr, local); \
	SET_PROC(globPtr, proc);  \
	SET_BOX(globPtr, box);    \
  } while (0)                       

#if 0
/* don't take a chance on the C inliner */

/* Accessor function to get the processor part */
INLINE_MODIFIER Process toproc(jGPointer globPtr)
INLINE({
	return GET_PROC(globPtr);
})

/* Accessor function to get the box part */
INLINE_MODIFIER Process tobox(jGPointer globPtr)
INLINE({
	return GET_BOX(globPtr);
})
#else
#define toproc(globPtr) 	GET_PROC(globPtr)
#define tobox(globPtr)		GET_BOX(globPtr)
#endif

#define toboxproc(globPtr) (*(jUIntPointer *)(&globPtr))
#define boxproctoproc(boxproc) (GET_PROC(*(jGPointer *)(&boxproc)))
#define boxproctobox(boxproc)  (GET_BOX(*(jGPointer *)(&boxproc)))

/* Accessor function to get the address part of a global pointer */
     /* Allow tolocal to be an lvalue */
#define tolocal(globPtr)  GET_ADDR(globPtr)

#define islocal(globPtr)   (GET_PROC(globPtr) == MYPROC)


/* ..........................
 *
 * Global pointer comparisons
 *
 * ..........................
 */

INLINE_MODIFIER int ___null_ptr_check(jGPointer gPtr)
INLINE ({
	return GET_ADDR(gPtr) == 0;
})

INLINE_MODIFIER int ___not_null_ptr_check(jGPointer gPtr)
INLINE ({
	return GET_ADDR(gPtr) != 0;
})

INLINE_MODIFIER int ___eq_ptr_check(jGPointer gPtr1, jGPointer gPtr2)
INLINE ({
	return ((GET_ADDR(gPtr1) == GET_ADDR(gPtr2)) &&
	        (GET_PROC(gPtr1) == GET_PROC(gPtr2)))
             || (___null_ptr_check(gPtr1) && ___null_ptr_check(gPtr2));
})

INLINE_MODIFIER int ___neq_ptr_check(jGPointer gPtr1, jGPointer gPtr2)
INLINE ({
	return GET_ADDR(gPtr1) != GET_ADDR(gPtr2)
	    || GET_PROC(gPtr1) != GET_PROC(gPtr2);

})


#endif /* __POINTERS_H */
