/* tic_am2_macros.h - Tic wrappers around AM entry points - 
 * all AM code should call these rather than calling AM directly 
 */
/* see copyright.txt for usage terms */

#ifndef _TIC_AM_H
#define _TIC_AM_H

#include <primitives.h>

/* ------------------------------------------------------------------------------------ */

#if !defined(COMM_AMUDP) && !defined(COMM_AMMPI)
/* Some questionable AM implementations (ahem, AMNOW) leave out some crucial entry points */

#define AM_Request1(endpoint, dest, handler, a) \
      AM_Request4(endpoint, dest, handler, (int)a, 0, 0, 0)
#define AM_Request2(endpoint, dest, handler, a, b) \
      AM_Request4(endpoint, dest, handler, (int)a, (int)b, 0, 0)
#define AM_Request3(endpoint, dest, handler, a, b, c) \
      AM_Request4(endpoint, dest, handler, (int)a, (int)b, (int)c, 0)
#define AM_Request5(endpoint, dest, handler, a, b, c, d, e) \
      AM_Request8(endpoint, dest, handler, (int)a, (int)b, (int)c, (int)d, \
				  (int)e, 0, 0, 0)
#define AM_Request6(endpoint, dest, handler, a, b, c, d, e, f) \
      AM_Request8(endpoint, dest, handler, (int)a, (int)b, (int)c, (int)d, \
				  (int)e, (int)f, 0, 0)
#define AM_Request7(endpoint, dest, handler, a, b, c, d, e, f, g) \
      AM_Request8(endpoint, dest, handler, (int)a, (int)b, (int)c, (int)d, \
				  (int)e, (int)f, (int)g, 0)
#define AM_Reply1(token, handler, a) \
      AM_Reply4(token, handler, (int)a, 0, 0, 0)
#define AM_Reply2(token, handler, a, b) \
      AM_Reply4(token, handler, (int)a, (int)b, 0, 0)
#define AM_Reply3(token, handler, a, b, c) \
      AM_Reply4(token, handler, (int)a, (int)b, (int)c, 0)
#define AM_Reply5(token, handler, a, b, c, d, e) \
      AM_Reply8(token, handler, (int)a, (int)b, (int)c, (int)d, (int)e, 0, 0, 0)
#define AM_Reply6(token, handler, a, b, c, d, e, f) \
      AM_Reply8(token, handler, (int)a, (int)b, (int)c, (int)d, (int)e, (int)f, 0, 0)
#define AM_Reply7(token, handler, a, b, c, d, e, f, g) \
      AM_Reply8(token, handler, (int)a, (int)b, (int)c, (int)d, (int)e, (int)f, (int)g, 0)

#endif

#ifdef COMM_AMVIA

#define AM_RequestXfer4(request_endpoint, reply_endpoint, dest_offset,   \
                        handler, source_addr, nbytes, arg0, arg1,        \
                        arg2, arg3)                                      \
        AM_RequestXfer8(request_endpoint, reply_endpoint, dest_offset,   \
                        handler, source_addr, nbytes, arg0, arg1, arg2,  \
                        arg3, 0, 0, 0, 0)

#define AM_ReplyXfer4(token, dest_offset, handler, source_addr,  \
                      nbytes, arg0, arg1, arg2, arg3)            \
        AM_ReplyXfer8(token, dest_offset, handler, source_addr,  \
                      nbytes, arg0, arg1, arg2, arg3, 0, 0, 0, 0)
#endif /* COMM_AMVIA */


#define AM_Safe(__function) 				                    \
do { 	int __rc; 							    \
	if (PREDICT_FALSE((__rc = __function) != AM_OK)) {                  \
		fprintf(stderr, "ERROR: %s failed with code %d at %s:%i\n", \
                #__function, __rc, __FILE__,__LINE__);                      \
	    abort(); 							    \
	} 								    \
  } while(0)

#define AM_Safe_Locked(__function) do { \
  __tic_lock(&AM_lock);                 \
  AM_Safe(__function);                  \
  __tic_unlock(&AM_lock);               \
  } while(0)

#define tic_poll() AM_Safe_Locked(AM_Poll(__sc_bundle))									  
/* ------------------------------------------------------------------------------------ */
#ifdef PTR64


#define TI_AM_Request1(a,b,c,d) \
     AM_Safe_Locked(AM_Request2(a, b, c, (int)HIGHWORD(d), (int)LOWWORD(d)))
#define TI_AM_Request1(a,b,c,d,e) \
     AM_Safe_Locked(AM_Request4(a, b, c, (int)HIGHWORD(d), (int)LOWWORD(d), \
                          (int)HIGHWORD(e), (int)LOWWORD(e)))

#define TI_AM_Request3(a,b,c,d,e,f) \
     AM_Safe_Locked(AM_Request6(a, b, c, (int)HIGHWORD(d), (int)LOWWORD(d), \
                 (int)HIGHWORD(e), (int)LOWWORD(e), \
                 (int)HIGHWORD(f), (int)LOWWORD(f)))

#define TI_AM_Request4(a,b,c,d,e,f,g) \
     AM_Safe_Locked(AM_Request8(a, b, c, (int)HIGHWORD(d), (int)LOWWORD(d), \
                 (int)HIGHWORD(e), (int)LOWWORD(e), \
                 (int)HIGHWORD(f), (int)LOWWORD(f), \
                 (int)HIGHWORD(g), (int)LOWWORD(g)))

#define TI_AM_RequestI2(a,b,c,d,e,f,g) \
     AM_Safe_Locked(AM_RequestI4(a, b, c, (int)HIGHWORD(d), (int)LOWWORD(d), \
                 (int)HIGHWORD(e), (int)LOWWORD(e), \
                 (int)HIGHWORD(f), (int)LOWWORD(f), \
                 (int)HIGHWORD(g), (int)LOWWORD(g)))

#define TI_AM_RequestI4(a,b,c,d,e,f,g,h,i) \
     AM_Safe_Locked(AM_RequestI8(a, b, c, (int)HIGHWORD(d), (int)LOWWORD(d), \
                 (int)HIGHWORD(e), (int)LOWWORD(e), \
                 (int)HIGHWORD(f), (int)LOWWORD(f), \
                 (int)HIGHWORD(g), (int)LOWWORD(g), \
                 (int)HIGHWORD(h), (int)LOWWORD(h), \
                 (int)HIGHWORD(i), (int)LOWWORD(i)))

#define TI_AM_Reply0(a, b)   \
     AM_Safe(AM_Reply0(a, b))

#define TI_AM_Reply1(a, b, d) \
     AM_Safe(AM_Reply2(a, b, (int)HIGHWORD(d), (int)LOWWORD(d)))

#define TI_AM_Reply2(a, b, d, e) \
     AM_Safe(AM_Reply4(a, b, (int)HIGHWORD(d), (int)LOWWORD(d), \
                     (int)HIGHWORD(e), (int)LOWWORD(e))

#define TI_AM_Reply3(a, b, d, e, f) \
     AM_Safe(AM_Reply6(a, b, (int)HIGHWORD(d), (int)LOWWORD(d), \
                 (int)HIGHWORD(e), (int)LOWWORD(e), \
                 (int)HIGHWORD(f), (int)LOWWORD(f)))

#define TI_AM_Reply4(a, b, d, e, f, g) \
     AM_Safe(AM_Reply8(a, b, (int)HIGHWORD(d), (int)LOWWORD(d), \
                 (int)HIGHWORD(e), (int)LOWWORD(e), \
                 (int)HIGHWORD(f), (int)LOWWORD(f), \
                 (int)HIGHWORD(g), (int)LOWWORD(g)))

#define TI_AM_ReplyI2(a, b, d, e, f, g) \
     AM_Safe(AM_ReplyI4(a, b, (int)HIGHWORD(d), (int)LOWWORD(d), \
                     (int)HIGHWORD(e), (int)LOWWORD(e), \
                     (int)HIGHWORD(f), (int)LOWWORD(f), \
                     (int)HIGHWORD(g), (int)LOWWORD(g)))

#define TI_AM_ReplyI4(a, b, d, e, f, g) \
     AM_Safe(AM_ReplyI8(a, b, (int)HIGHWORD(d), (int)LOWWORD(d), \
                     (int)HIGHWORD(e), (int)LOWWORD(e), \
                     (int)HIGHWORD(f), (int)LOWWORD(f), \
                     (int)HIGHWORD(g), (int)LOWWORD(g), \
                     (int)HIGHWORD(h), (int)LOWWORD(h), \
                     (int)HIGHWORD(i), (int)LOWWORD(i)))

#define TI_AM_RequestXfer0(a, b, c, d, e, f) \
     AM_Safe_Locked(AM_RequestXfer4(a, b, c, d, e, (int)f, 0, 0, 0, 0))

#define TI_AM_RequestXfer1(a, b, c, d, e, f, g) \
     AM_Safe_Locked(AM_RequestXfer4(a, b, c, d, e, f, \
                     (int)HIGHWORD(g), (int)LOWWORD(g), 0, 0))

#define TI_AM_ReplyXfer1(token, dest_offset, handler, src_addr, len, f) \
     AM_Safe(AM_ReplyXfer4(token, dest_offset, handler, (void *)src_addr, len, \
                     (int)HIGHWORD(f), (int)LOWWORD(f), 0, 0))


/* ------------------------------------------------------------------------------------ */
/* ^^^ PTR64  ,,, PTR32 */
#else

#define TI_AM_Request1(a, b, c, d) \
     AM_Safe_Locked(AM_Request1(a, b, c, d))

#define TI_AM_Request2(a, b, c, d, e)  \
     AM_Safe_Locked(AM_Request2(a, b, c, d, e))

#define TI_AM_Request3(a, b, c, d, e, f) \
     AM_Safe_Locked(AM_Request3(a, b, c, d, e, f))

#define TI_AM_Request4(a, b, c, d, e, f, g)  \
     AM_Safe_Locked(AM_Request4(a, b, c, d, e, f, g))

#define TI_AM_Request5(a, b, c, d, e, f, g, h) \
     AM_Safe_Locked(AM_Request5(a, b, c, d, e, f, g, h))

#define TI_AM_RequestI2(a, b, c, d, e, f, g)  \
     AM_Safe_Locked(AM_RequestI4(a, b, c, d, e, (int)f, (int)g, 0, 0))

#define TI_AM_RequestI4(a, b, c, d, e, f, g, h, i)  \
     AM_Safe_Locked(AM_RequestI4(a, b, c, d, e, (int)f, (int)g, (int)h, (int)i))

#define TI_AM_RequestI8(a, b, c, d, e, f, g, h, i, j, k, l, m)  \
     AM_Safe_Locked(AM_RequestI8(a, b, c, d, e, (int)f, (int)g, (int)h, (int)i, (int)j, (int)k, (int)l, (int)m))

#define TI_AM_Reply0(a, b)    AM_Safe(AM_Reply0(a, b))
#define TI_AM_Reply1(a, b, d) AM_Safe(AM_Reply1(a, b, d))
#define TI_AM_Reply2(a, b, d, e) AM_Safe(AM_Reply2(a, b, d, e))
#define TI_AM_Reply3(a, b, d, e, f) AM_Safe(AM_Reply3(a, b, d, e, f))
#define TI_AM_Reply4(a, b, d, e, f, g) AM_Safe(AM_Reply4(a, b, d, e, f, g))

#define TI_AM_ReplyI2(a, b, d, e, f, g) AM_Safe(AM_ReplyI4(a, b, d, e, (int)f, (int)g, 0, 0))
#define TI_AM_ReplyI4(a, b, d, e, f, g, h, i) AM_Safe(AM_ReplyI4(a, b, d, e, (int)f, (int)g, (int)h, (int)i))

#define TI_AM_RequestXfer0(a, b, c, d, e, f) \
     AM_Safe_Locked(AM_RequestXfer4(a, b, c, d, e, f, 0, 0, 0, 0))

#define TI_AM_RequestXfer1(a, b, c, d, e, f, g) \
     AM_Safe_Locked(AM_RequestXfer4(a, b, c, d, e, f, g, 0, 0, 0))


#define TI_AM_ReplyXfer1(a, b, c, d, e, f) \
     AM_Safe(AM_ReplyXfer4(a, b, c, (void *)d, e, f, 0, 0, 0))

#endif /* PTR64 */

#endif /* _TIC_AM_H */

