#ifndef _INCLUDE_MEM_WIDE_H_
#define _INCLUDE_MEM_WIDE_H_

#define WIDE_POINTERS

#include "instance_header.h"
#include "glob_ptr_base.h"
#include "mem-local.h"

/* PLASSIGN_GLOBAL is illegal. */
/* WEAK_PLASSIGN_GLOBAL is illegal. */

#ifdef NOMEMCHECKS
#  define _PGASSIGN_GLOBAL(ptr, val, primtype )
  do { ASSIGN_GLOBAL_ ## primtype (ptr, val); } while (0)
#  define _WEAK_PGASSIGN_GLOBAL(ptr, val, primtype ) \
  do { WEAK_ASSIGN_GLOBAL_ ## primtype (ptr, val); } while (0)
#else /* NOMEMCHECKS */
#  define _PGASSIGN_GLOBAL(ptr, val, primtype ) \
  do { if (!(!isLocal(val) || SHARED_REGIONID(PL2RegionId(TO_LOCAL(val)))) && \
	   (!isLocal(ptr) || SHARED_REGIONID(PL2RegionId(TO_LOCAL(ptr))))) \
	   sharingViolation(); \
	else ASSIGN_GLOBAL_ ## primtype (ptr, val); } while (0)
#  define _WEAK_PGASSIGN_GLOBAL(ptr, val, primtype ) \
  do { if (!(!isLocal(val) || SHARED_REGIONID(PL2RegionId(TO_LOCAL(val)))) && \
	   (!isLocal(ptr) || SHARED_REGIONID(PL2RegionId(TO_LOCAL(ptr))))) \
	   sharingViolation(); \
	else WEAK_ASSIGN_GLOBAL_ ## primtype (ptr, val); } while (0)
#endif /* NOMEMCHECKS */

/* DOB: see mem-local.h for the naming scheme on these macros
   the prefixes constrain which value types are being accessed,
   so the type suffixes not shown are not meaningful and will never be generated
 */
#define PGASSIGN_GLOBAL_gp(ptr, val) _PGASSIGN_GLOBAL(ptr, val, gp)

#define WEAK_PGASSIGN_GLOBAL_gp(ptr, val) _WEAK_PGASSIGN_GLOBAL(ptr, val, gp)

#define _SPLASSIGN_GLOBAL(ptr, val, primtype)	ASSIGN_GLOBAL_ ## primtype (ptr, val)
#define _SPGASSIGN_GLOBAL(ptr, val, primtype)	ASSIGN_GLOBAL_ ## primtype (ptr, val)
#define _WEAK_SPLASSIGN_GLOBAL(ptr, val, primtype) WEAK_ASSIGN_GLOBAL_ ## primtype (ptr, val)	
#define _WEAK_SPGASSIGN_GLOBAL(ptr, val, primtype) WEAK_ASSIGN_GLOBAL_ ## primtype (ptr, val)	


#define SPGASSIGN_GLOBAL_gp(ptr, val) _SPGASSIGN_GLOBAL(ptr, val, gp)

#define WEAK_SPGASSIGN_GLOBAL_gp(ptr, val) _WEAK_SPGASSIGN_GLOBAL(ptr, val, gp)

#define SPLASSIGN_GLOBAL_lp(ptr, val) _SPLASSIGN_GLOBAL(ptr, val, lp)

#define WEAK_SPLASSIGN_GLOBAL_lp(ptr, val) _WEAK_SPLASSIGN_GLOBAL(ptr, val, lp)


#define TO_LOCAL(ptr)  GET_ADDR(ptr) /* DOB: it's crucial we don't destroy pointer type */
#define TO_PROC(ptr)   toproc(CAST_GPTR(ptr))
#define TO_BOX(ptr)    tobox(CAST_GPTR(ptr))

#define TO_GLOBAL( global, processor, address )	 mutateglobal(CAST_GPTR((global)), (processor), (address))
#define TO_GLOBALWITHBOX( global, box, processor, address) \
   mutateglobalwithbox(CAST_GPTR(global), proc, box, address)

#define FIELD_ADDR_GLOBAL(val, obj, fld)	TO_GLOBAL(val, TO_PROC(obj), (&obj.addr->fld))
#define IFACE_DISPATCH_GLOBAL(ci, mid)          IFACE_DISPATCH_LOCAL(ci, mid)
#define INDEX_GLOBAL(element, base, offset)	TO_GLOBAL(element, TO_PROC(base), (base.addr + offset))
#define SUM_GLOBAL(type, element, base, offset)	TO_GLOBAL(element, TO_PROC(base), (type) ((char *)(TO_LOCAL(base)) + offset))
#define EQUAL_GLOBAL(p, q)			((isNull(p) && isNull(q)) || ((TO_BOX(p) == TO_BOX(q)) && (TO_LOCAL(p) == TO_LOCAL(q))))

#define CLASS_INFO_GLOBAL( val, type, obj )		\
  do {							\
    PL ## type field_;					\
    FIELD_ADDR_GLOBAL( field_, obj, class_info );	\
    DEREF_GLOBAL_lp( val, field_ );			\
  } while (0)

/* DOB: changed to match lang ref. defn of desmenses on 10/22 */
#define isLocal( global )	(TO_BOX(global) == MYBOX)

#define isDirectlyAddressable( global )	(TO_BOX(global) == MYBOX)

#define globalize( global, local )	TO_GLOBAL( global, MYPROC, local )

#define localize( local, global )				\
  do {								\
    if (isLocal(global))					\
      (local) = TO_LOCAL( global );				\
    else							\
      m23throwClassCastExceptionmT11NativeUtils4lang2ti();	\
  } while (0)


#define local_to_global_copy( local, global, count ) ASSIGN_GLOBAL_anonymous_bulk(global, count, local)
#define global_to_local_copy( global, local, count ) DEREF_GLOBAL_anonymous_bulk(local, count, global)




#endif /* !_INCLUDE_MEM_WIDE_H_ */
