#ifndef _INCLUDE_TLIB_NATIVE_UTILS_H_
#define _INCLUDE_TLIB_NATIVE_UTILS_H_

#include "linkage.h"
#include "mem-common.h"
#include "primitives.h"

#define _TOSTR_HELP(x) #x
#define _TOSTR(x) _TOSTR_HELP(x)

/* __TI_CURRENT_FUNCTION__ is re-defined for code-genned methods to a demangled
 name, but try to get one from the C compiler for native code */
#ifdef __GNUC__
  /* __func__ should also work for ISO C99 compilers */
  #define __TI_CURRENT_FUNCTION__ __FUNCTION__
#else
  #define __TI_CURRENT_FUNCTION__ ""
#endif
extern char *build_loc_str(const char *funcname, const char *filename, int linenum);
#define __current_loc  build_loc_str(__TI_CURRENT_FUNCTION__,__FILE__,__LINE__)

#define CHECK_NULL_GLOBAL( object )				\
  do {								\
    if (PREDICT_FALSE(isNull(object)))				\
      tossNullPointerException_str(__current_loc);		\
  } while(0)


#define CHECK_NULL_LOCAL( object )				\
  do {								\
    if (PREDICT_FALSE(!(object)))				\
      tossNullPointerException_str(__current_loc);		\
  } while(0)

void tossNullPointerException_str(char * const string) __attribute__((noreturn)
);
 
void tossMonitorStateException_str(char * const string) __attribute__((noreturn));

void tossArrayStoreException_str(char * const string) __attribute__((noreturn));

void tossIllegalArgumentException_str(char * const string) __attribute__((noreturn));


#endif
