#include <limits.h>
#include <pwd.h>
#include <stdlib.h>
#include <sys/time.h>
#include <unistd.h>

#include <class_header.h>
#include <java_string.h>
#include <native-stubs.h>
#include <native-utils.h>
#include "runtime-options.h"
#include "layout!PTAjbyte.h"
#include "layout!PLti5cdescmT6Object4lang4java.h"
#include "layout!Pclass_header.h"
#include "layout!Pjbyte.h"
#include "layout!Pjint.h"
#include "layout!TAjbyte.h"

extern char *RUNTIME_MODEL;

static void checkIsArray( PT6Object4lang4java object )
{
    typedef GP_type( TypeCategory ) GP_TypeCategory;

    ti5cdescmT6Object4lang4java *info;
    Pclass_header header;
    GP_TypeCategory categoryField;
    TypeCategory category;

    CLASS_INFO_GLOBAL( info, ti5cdescmT6Object4lang4java, object );
    TO_GLOBAL( header, TO_PROC( object ), (class_header *) info );
    FIELD_ADDR_GLOBAL( categoryField, header, category );
    FENCE_PRE_READ();
    /* MUST use _bulk here because width of TypeCategory might not always be sizeof(jint) */
    DEREF_GLOBAL_bulk( category, categoryField ); 

    if (category != JavaArray) tossArrayStoreException_str("argument to System.arraycopy() is not an array!");
}
  

void m9arraycopyPT6Object4lang4javaIPT6Object4lang4javaIImT6System4lang4java(
    PT6Object4lang4java src,
    jint src_pos,
    PT6Object4lang4java dst,
    jint dst_pos,
    jint length)
{
    PTAjbyte src_array, dst_array;
    jint src_size, dst_size;
    jint src_isatomic, dst_isatomic;
    Pjbyte src_data, dst_data;

    CHECK_NULL_GLOBAL(src);
    CHECK_NULL_GLOBAL(dst);

    checkIsArray( src );
    checkIsArray( dst );

    src_array = * (PTAjbyte *) &src;
    dst_array = * (PTAjbyte *) &dst;

    JAVA_ARRAY_SIZE_GLOBAL( src_size, src_array );
    JAVA_ARRAY_SIZE_GLOBAL( dst_size, dst_array );
    JAVA_ARRAY_ISATOMIC_GLOBAL( src_isatomic, src_array );
    JAVA_ARRAY_ISATOMIC_GLOBAL( dst_isatomic, dst_array );
    if ( src_size != dst_size || !!src_isatomic != !!dst_isatomic) 
      tossArrayStoreException_str("array arguments to System.arraycopy() have incompatible element type");

    /* TODO: PR 349 still need to check that:
     *  1. primitive element types with same width are equal
     *  2. reference element types are assignment-compatible, and each element is assignment-compatible
     */

  #ifndef NOMEMCHECKS
    if (!src_isatomic) { /* region safety check */
      if (!SHARED_REGIONID(PG2RegionId(src)) && 
           SHARED_REGIONID(PG2RegionId(dst))) 
        tossArrayStoreException_str("System.arraycopy() may not copy arrays with non-atomic element type"
                                    " from private region to shared region");
    }
  #endif

    JAVA_ARRAY_CHECK_GLOBAL( src_array, src_pos, length );
    JAVA_ARRAY_CHECK_GLOBAL( dst_array, dst_pos, length );

    /* We would like to use JAVA_ARRAY_ADDR_GLOBAL for the
       indexing computations below, but we cannot.  The arrays
       might not really be arrays of bytes, and so the bounds
       check inside JAVA_ARRAY_ADDR_GLOBAL will fail
       inappropriately.
       */
    
    FIELD_ADDR_GLOBAL( src_data, src_array, data[0] );
    INDEX_GLOBAL( src_data, src_data, src_pos * src_size );
    
    FIELD_ADDR_GLOBAL( dst_data, dst_array, data[0] );
    INDEX_GLOBAL( dst_data, dst_data, dst_pos * dst_size );

  {
    int copy_sz = src_size * length;
    int islocalsrc = isDirectlyAddressable(src_data);
    int islocaldest = isDirectlyAddressable(dst_data);

    if (copy_sz == 0) return; /* a stupid case that actually happens */

    if (islocalsrc && islocaldest) {
      jbyte *localdest = TO_LOCAL(dst_data);
      jbyte *localsrc = TO_LOCAL(src_data);
      int has_overlap = (localdest >= localsrc && localdest < localsrc + copy_sz) ||
                        (localsrc >= localdest && localsrc < localdest + copy_sz);
      if (localdest == localsrc) return; /* a stupid case that actually happens */
      if (has_overlap)
        memmove(localdest, localsrc, copy_sz);
      else 
        memcpy(localdest, localsrc, copy_sz);
    }
    else if (islocalsrc && !islocaldest) {
      jbyte *localsrc = TO_LOCAL(src_data);
      if (!src_isatomic) {
        /* we may have some escaping pointers - inform distributed GC about them */
        if (src_size == sizeof(jGPointer)) { /* elements are gp's */
          GC_PTR_ESC(localsrc, length);
        }
        else { /* elements have embedded gp's */
          /* TODO: use array type info to only pick up the real pointers */
          GC_PTR_ESC_ALL_GPS(localsrc, copy_sz);
        }
      }
      local_to_global_copy(localsrc, dst_data, copy_sz);
    }
    else {
      #if defined(USE_DISTRIBUTED_GC) && !defined(USE_GC_NONE)
      if (TO_BOX(src_array) != TO_BOX(dst_array) && !src_isatomic) {
          if (src_size == sizeof(jGPointer)) { /* elements are gp's */
            /* TODO: "pulling" some global pointers - inform source they escaped 
             *  this is currently taken care of conservatively by the AM request handler for bulk gets
             *  but we may be able to improve that somewhat with the type info available here
             */
          }
          else {
            /* TODO: "pulling" some embedded global pointers - inform source they escaped 
             *  this is currently taken care of conservatively by the AM request handler for bulk gets
             *  but we may be able to improve that somewhat with the type info available here
             */
          }
      }
      #endif

      if (!islocalsrc && islocaldest) {
        if (src_isatomic)
          ti_bulk_read_noptrs(TO_LOCAL(dst_data), TO_PROC(src_data), TO_LOCAL(src_data), copy_sz);
        else
          ti_bulk_read(TO_LOCAL(dst_data), TO_PROC(src_data), TO_LOCAL(src_data), copy_sz);
      }
      else { /* !islocalsrc && !islocaldest */
        VARARRAY_DECLARE(jbyte, tmp_buf, copy_sz);
        VARARRAY_CREATE(jbyte, tmp_buf, copy_sz);
        if (src_isatomic)
          ti_bulk_read_noptrs(tmp_buf, TO_PROC(src_data), TO_LOCAL(src_data), copy_sz);
        else
          ti_bulk_read(tmp_buf, TO_PROC(src_data), TO_LOCAL(src_data), copy_sz);
        local_to_global_copy(tmp_buf, dst_data, copy_sz);
        VARARRAY_DESTROY(tmp_buf);
      }
    }
  }
}


jlong m17currentTimeMillismT6System4lang4java()
{
    struct timeval now;
    assert(!gettimeofday(&now, 0));
    
    /* We could round the usecs, but it does not seem worth it. */
    return now.tv_sec * (jlong) 1000 + now.tv_usec / 1000;
}

static void addProperty( PT10Properties4util4java props, char keyC[], char valueC[] )
{
  if (valueC)
    {
      const LP_JString key = java_string_build_8( keyC );
      const LP_JString value = java_string_build_8( valueC );
      m11addPropertyPT10Properties4util4javaLT6String4lang4javaLT6String4lang4javamT11NativeUtils4lang2ti( props, key, value );
    }
}    
    
extern int bounds_checking;
extern const char *tcCompilerFlags;

PT10Properties4util4java
m14initPropertiesPT10Properties4util4javamT6System4lang4java( PT10Properties4util4java props )
{
  char cwdbuf[PATH_MAX+2];
  char * const cwd = getcwd( cwdbuf, PATH_MAX );
  #if 0 
    /* this is unthreadsafe and has been observed to cause deadlocks on Linux
     * it also causes things to be allocated using malloc, which may piss off 
     * the GC
     */
    const struct passwd * const user = getpwuid( getuid() );
  #else
    /* a non-standard but common (and safe) option */
    char pwbuf_data[1024];
    struct passwd pwbuf_struct;
    struct passwd * user = NULL;
    /* getpwuid_r is supposed to be re-entrant, but at least on Linux it 
     * clearly is not, so enforce a critical section around it
     */
    static __tic_lock_t getpwuid_r_lock = __tic_lock_decl_initializer;
    __tic_lock(&getpwuid_r_lock);
    #if defined(SOLARIS) && !defined(_POSIX_PTHREAD_SEMANTICS)
      user = getpwuid_r( getuid(), &pwbuf_struct, pwbuf_data, 1024 );
    #else
      getpwuid_r( getuid(), &pwbuf_struct, pwbuf_data, 1024, &user );
    #endif
    __tic_unlock(&getpwuid_r_lock);
  #endif

  addProperty( props, "java.version",		VERSION );
  addProperty( props, "java.vm.version",	VERSION );
  addProperty( props, "java.specification.name",		"Java Platform API Specification" );
  addProperty( props, "java.specification.version",		"Sun Microsystems Inc." );
  addProperty( props, "java.specification.version",		"1.0" );
  addProperty( props, "java.vendor",		"Titanium Project" );
  addProperty( props, "java.vendor.url",	"http://titanium.cs.berkeley.edu/" );
  addProperty( props, "java.home",		"/" );
  addProperty( props, "java.class.version",	"0" );
  addProperty( props, "java.class.path",	"/" );
  addProperty( props, "os.name",		"Titanium" );
  addProperty( props, "os.arch",		"Titanium" );
  addProperty( props, "os.version",		"0" );
  addProperty( props, "file.separator",	"/" );
  addProperty( props, "path.separator",	":" );
  addProperty( props, "line.separator",	"\n" );
  addProperty( props, "runtime.model", RUNTIME_MODEL);
  #ifdef MEMORY_DISTRIBUTED
    addProperty( props, "runtime.distributed", "true");
  #else
    addProperty( props, "runtime.distributed", "false");
  #endif

  #ifdef MEMORY_SHARED
    addProperty( props, "runtime.shared", "true");
  #else
    addProperty( props, "runtime.shared", "false");
  #endif

  if (bounds_checking) 
    addProperty( props, "runtime.boundschecking", "true");
  else
    addProperty( props, "runtime.boundschecking", "false");

#if !defined(USE_GC_NONE)
  if (!GC_dont_gc) addProperty( props, "runtime.gc", "true");
  else
#endif
    addProperty( props, "runtime.gc", "false"); /* note: keep this line with above */

  addProperty( props, "compiler.flags", (char*)tcCompilerFlags);

  if (user) {
    addProperty( props, "user.name", user->pw_name );
    addProperty( props, "user.home", user->pw_dir );
  }
    
  if (cwd)
    addProperty( props, "user.dir", cwd );

  return props;
}
