#ifndef _STOPTIFU_RUNTIME_H_
#define _STOPTIFU_RUNTIME_H_

#include <runtime-options-var.h>
#include <ti-gc.h>

/****************************************************************************
 * Create macros for using alloca, if available.
 ****************************************************************************/

#if defined(USE_RUNTIME_VARARRAY) && !defined(FORCE_NO_RUNTIME_VARARRAY) && HAVE_ALLOCA_H && ALLOCA_IN_C
#define STOPTIFU_VARARRAY_CREATE(type, var, size) \
    do { var = (type *) alloca(sizeof(type)*(size)); } while(0)
#define STOPTIFU_VARARRAY_DESTROY(var) 
#else /* USE_RUNTIME_VARARRAY && !FORCE_NO_RUNTIME_VARARRAY */
#define STOPTIFU_VARARRAY_CREATE(type, var, size) var = (type *) ti_malloc(sizeof(type)*(size))
#define STOPTIFU_VARARRAY_DESTROY(var) ti_free(var)
#endif /* USE_RUNTIME_VARARRAY && !FORCE_NO_RUNTIME_VARARRAY */

/****************************************************************************
 * Control `extern' and `inline' (copied from tic.h)
 ****************************************************************************/

/* Usage: 
 * The proper (portable) way to declare inline functions in
 * stoptifu headers is:
 *
 *   INLINE_MODIFIER int foo(int bar) 
 *   INLINE ({
 *     function body...
 *     })
 */

/* start with a clean slate */
#ifdef EXTERN
#undef EXTERN
#endif
#ifdef INLINE
#undef INLINE
#endif

/* Defining STOPTIFU_DEBUG has the effect of disabling all inlining of functions
 * which helps when debugging. */
#if defined(STOPTIFU_DEBUG)
#	define	INLINE(code) ;
#else
#	define	INLINE(code) {code}
#endif

/* Defining STOPTIFU_INTERNAL_DEFINE has the effect of producing a
 * real (non-extern) version of all functions and variables. 
 *
 * Exactly one .c file should define this in the distribution to 
 * support the STOPTIFU_DEBUG option and non-GNU compilers
 */
#undef FORCE_STATIC
#ifdef STOPTIFU_INTERNAL_DEFINE
#      define EXTERN
#      undef INLINE
#      define INLINE(code) {code}
#      define FORCE_NO_RUNTIME_INLINE
#      define FORCE_STATIC 0
#else
#      define EXTERN extern
#      define FORCE_STATIC 1
#endif

/* get definition for INLINE_MODIFIER, INLINE_MODIFIER_FOR_SOLARIS_AM */
#include "runtime-options-inl.h"


#include <values.h>
#include <stoptifu/alloc.h>
#include <stoptifu/ivseti.h>
#include <stoptifu/overlap.h>

#ifndef COMPUTE_ALPHA_DEBUG_LEVEL
#define COMPUTE_ALPHA_DEBUG_LEVEL 0
#endif

#ifndef STOPTIFU_STATS
#if DEBUG_TILE_EXECUTION
#define STOPTIFU_STATS 1
#else
#define STOPTIFU_STATS 0
#endif
#endif

extern ivseti *compute_alpha(int n, const int *p, const int * const *v,
			     const int * const *inv, int k, int **r, int dbg);

#endif
