/* Linked list type */

/* User should #define:
   T
   list
   cons
   head
   tail
   length
   freecell
   set_head
   set_tail
   dreverse
   dreverse_and_extend
*/

typedef struct list {
  T head;
  struct list *tail;
} list;

INLINE_MODIFIER list *cons(T head, list *tail) INLINE ({
  list *result = ALLOC(list, 1);
  result->head = head;
  result->tail = tail;
  return result;
})

INLINE_MODIFIER T head(list *l) INLINE ({ return l->head; })
INLINE_MODIFIER list *tail(list *l) INLINE ({ return l->tail; })

INLINE_MODIFIER list *freecell(list *l)
INLINE ({ list *r = l->tail; FREE(l); return r; })

INLINE_MODIFIER void set_head(list *l, T h)
INLINE ({ l->head = h; })
INLINE_MODIFIER void set_tail(list *l, list *t)
INLINE ({ l->tail = t; })
  
INLINE_MODIFIER int length(list *l)
INLINE ({
  int i = 0;
  while (l != NULL) {
    i++;
    l = l->tail;
  }
  return i;
})

/* Destructively reverse l and destructively append p to the result. */
INLINE_MODIFIER list *dreverse_and_extend(list *l, list *p)
INLINE ({
  if (l == NULL)
    return p;
  while (l->tail != NULL) {
    list *next = l->tail;
    l->tail = p;
    p = l;
    l = next;
  }
  l->tail = p;
  return l;
})

INLINE_MODIFIER list *dreverse(list *l)
INLINE ({
  return dreverse_and_extend(l, NULL);
})

